﻿using System.Collections.Generic;
using System.Drawing;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding.Views;
using HeuristicLab.Visualization;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Views {
  public partial class SymbolicDataAnalysisExpressionLineageExplorerChart : ChartControl {
    public SymbolicDataAnalysisExpressionLineageExplorerChart() {
      InitializeComponent();
      if (Chart == null) { Chart = new Chart(0, 0, PreferredSize.Width, PreferredSize.Height); }
    }

    public void Clear() {
      Chart.Group.Clear();
    }

    public void Add(IPrimitive primitive) {
      Chart.Group.Add(primitive);
    }

    public void AddRange(IEnumerable<IPrimitive> primitives) {
      Chart.Group.AddRange(primitives);
    }

    public void Draw(Graphics graphics) {
      Chart.Group.Draw(graphics);
    }

    public bool UpdateEnabled {
      get { return Chart.UpdateEnabled; }
      set { Chart.UpdateEnabled = value; }
    }

    public void EnforceUpdate() {
      Chart.EnforceUpdate();
    }

    public void FlipVertical() {
      foreach (var primitive in Chart.Group.Primitives) {
        var tile = primitive as SymbolicExpressionTreeTile;
        if (tile != null) {
          foreach (var p in tile.Primitives) {
            FlipPrimitive(p);
          }
        } else {
          FlipPrimitive(primitive);
        }
      }
    }

    public void FlipPrimitive(IPrimitive primitive) {
      var height = pictureBox.Height;
      var rect = primitive as RectangularPrimitiveBase;
      var line = primitive as LinearPrimitiveBase;
      if (rect != null) {
        rect.SetPosition(rect.LowerLeft.X, height - rect.UpperRight.Y, rect.UpperRight.X, height - rect.LowerLeft.Y);
      } else if (line != null) {
        line.SetPosition(line.Start.X, height - line.Start.Y, line.End.X, height - line.End.Y);
      }
    }
  }
}
